pub struct Slot<V> { /* private fields */ }Implementations
Sourceimpl<V> Slot<V>
impl<V> Slot<V>
pub fn generation(&self) -> u32
pub fn value_ptr(&self) -> *mut V
Sourcepub unsafe fn value_unchecked(&self) -> &V
pub unsafe fn value_unchecked(&self) -> &V
Safety
The value must be initialized. You can use generation to determine the state of the
slot.
Sourcepub unsafe fn value_unchecked_mut(&mut self) -> &mut V
pub unsafe fn value_unchecked_mut(&mut self) -> &mut V
Safety
The value must be initialized. You can use generation to determine the state of the
slot.