pub struct CollectorHandle { /* private fields */ }Implementations
Sourceimpl CollectorHandle
impl CollectorHandle
pub fn new() -> Self
Sourcepub unsafe fn pin(&self) -> Guard<'_>
pub unsafe fn pin(&self) -> Guard<'_>
Safety
The returned Guard must not outlive any collection that uses this collector. It would
result in the Guard’s drop implementation attempting to free slots from the already freed
collection, resulting in a Use-After-Free. You must ensure that the Guard has its
lifetime bound to the collections it protects.